# Copyright 2017-2021 Orbital Insight Inc., all rights reserved.
# Contains confidential and trade secret information.
# Government Users:  Commercial Computer Software - Use governed by
# terms of Orbital Insight commercial license agreement.


# TODO:
# add retry
# handle exceptions
# add custom logic (request_id, ....)

import grpc

from terrascope_api import models
from terrascope_api.stubs import system_pb2_grpc as system_pb2_grpc
from terrascope_api.stubs import aoi_pb2_grpc as aoi_pb2_grpc
from terrascope_api.stubs import aoi_collection_pb2_grpc as aoi_collection_pb2_grpc
from terrascope_api.stubs import aoi_version_pb2_grpc as aoi_version_pb2_grpc
from terrascope_api.stubs import aoi_transaction_pb2_grpc as aoi_transaction_pb2_grpc
from terrascope_api.stubs import algorithm_computation_pb2_grpc as algorithm_computation_pb2_grpc
from terrascope_api.stubs import visualization_pb2_grpc as visualization_pb2_grpc
from terrascope_api.stubs import user_pb2_grpc as user_pb2_grpc
from terrascope_api.stubs import user_collection_pb2_grpc as user_collection_pb2_grpc
from terrascope_api.stubs import token_pb2_grpc as token_pb2_grpc
from terrascope_api.stubs import permission_pb2_grpc as permission_pb2_grpc
from terrascope_api.stubs import algorithm_pb2_grpc as algorithm_pb2_grpc
from terrascope_api.stubs import algorithm_version_pb2_grpc as algorithm_version_pb2_grpc
from terrascope_api.stubs import algorithm_config_pb2_grpc as algorithm_config_pb2_grpc
from terrascope_api.stubs import analysis_pb2_grpc as analysis_pb2_grpc
from terrascope_api.stubs import analysis_version_pb2_grpc as analysis_version_pb2_grpc
from terrascope_api.stubs import analysis_config_pb2_grpc as analysis_config_pb2_grpc
from terrascope_api.stubs import analysis_computation_pb2_grpc as analysis_computation_pb2_grpc
from terrascope_api.stubs import toi_pb2_grpc as toi_pb2_grpc
from terrascope_api.stubs import result_pb2_grpc as result_pb2_grpc
from terrascope_api.stubs import data_source_pb2_grpc as data_source_pb2_grpc
from terrascope_api.stubs import data_type_pb2_grpc as data_type_pb2_grpc
from terrascope_api.stubs import credit_pb2_grpc as credit_pb2_grpc


# todo bring up  to date with async client or drop

class TerrascopeSyncClient:
    def __init__(self, oi_papi_url, port=80, secure=False):
        self.api = TerrascopeSyncApi(oi_papi_url, port, secure)
        self.models = models


class TerrascopeSyncApi:
    def __init__(self, oi_papi_url, port=80, secure=False):
        self._oi_papi_url = oi_papi_url
        self._port = port
        self._channel = self._get_secure_channel() if secure else self._get_insecure_channel()
        self.system = system_pb2_grpc.SystemApiStub(self._channel)
        self.aoi = aoi_pb2_grpc.AOIApiStub(self._channel)
        self.aoi_collection = aoi_collection_pb2_grpc.AOICollectionApiStub(self._channel)
        self.aoi_version = aoi_version_pb2_grpc.AOIVersionApiStub(self._channel)
        self.aoi_transaction = aoi_transaction_pb2_grpc.AOITransactionApiStub(self._channel)
        self.computation = algorithm_computation_pb2_grpc.AlgorithmComputationApiStub(self._channel)
        self.visualization = visualization_pb2_grpc.VisualizationApiStub(self._channel)
        self.permission = permission_pb2_grpc.PermissionApiStub(self._channel)
        self.user = user_pb2_grpc.UserApiStub(self._channel)
        self.user_collection = user_collection_pb2_grpc.UserCollectionApiStub(self._channel)
        self.token = token_pb2_grpc.TokenApiStub(self._channel)
        self.algorithm = algorithm_pb2_grpc.AlgorithmApiStub(self._channel)
        self.algorithm_version = algorithm_version_pb2_grpc.AlgorithmVersionApiStub(self._channel)
        self.algorithm_config = algorithm_config_pb2_grpc.AlgorithmConfigApiStub(self._channel)
        self.analysis = analysis_pb2_grpc.AnalysisApiStub(self._channel)
        self.analysis_version = analysis_version_pb2_grpc.AnalysisVersionApiStub(self._channel)
        self.toi = toi_pb2_grpc.TOIApiStub(self._channel)
        self.result = result_pb2_grpc.ResultApiStub(self._channel)
        self.credit = credit_pb2_grpc.CreditApiStub(self._channel)
        self.analysis_config = analysis_config_pb2_grpc.AnalysisConfigApiStub(self._channel)
        self.analysis_computation = analysis_computation_pb2_grpc.AnalysisComputationApiStub(self._channel)
        self.data_source = data_source_pb2_grpc.DataSourceAPIStub(self._channel)
        self.data_type = data_type_pb2_grpc.DataTypeAPIStub(self._channel)

    def _get_insecure_channel(self):
        return grpc.insecure_channel(f"{self._oi_papi_url}:{self._port}",
                                     options=[('grpc.max_send_message_length', -1),
                                              ('grpc.max_receive_message_length', -1)]
                                     )

    def _get_secure_channel(self):
        creds = grpc.ssl_channel_credentials(root_certificates=None, private_key=None, certificate_chain=None)
        return grpc.secure_channel(f"{self._oi_papi_url}:{self._port}", creds, compression=None,
                                   options=[('grpc.max_send_message_length', -1),
                                            ('grpc.max_receive_message_length', -1)])

    def __del__(self):
        self._channel.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: system.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0csystem.proto\x12\x07oi.papi\x1a\x1cgoogle/api/annotations.proto\"\r\n\x0bPingRequest\"#\n\x0cPingResponse\x12\x13\n\x0bstatus_code\x18\x01 \x01(\r2a\n\tSystemApi\x12T\n\x08get_ping\x12\x14.oi.papi.PingRequest\x1a\x15.oi.papi.PingResponse\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/systemapi/get_pingb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'system_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SYSTEMAPI.methods_by_name['get_ping']._options = None
  _SYSTEMAPI.methods_by_name['get_ping']._serialized_options = b'\202\323\344\223\002\025\022\023/systemapi/get_ping'
  _PINGREQUEST._serialized_start=55
  _PINGREQUEST._serialized_end=68
  _PINGRESPONSE._serialized_start=70
  _PINGRESPONSE._serialized_end=105
  _SYSTEMAPI._serialized_start=107
  _SYSTEMAPI._serialized_end=204
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x64\x61ta_type.proto\x12\x07oi.papi\"k\n\x08\x44\x61taType\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\x12\x17\n\x0f\x64\x61ta_source_ids\x18\x04 \x03(\t\x12\x13\n\x0bsensor_type\x18\x05 \x01(\t\"_\n\x15\x44\x61taTypeCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\x12\x13\n\x0bsensor_type\x18\x04 \x01(\t\"S\n\x16\x44\x61taTypeCreateResponse\x12\x13\n\x0bstatus_code\x18\x01 \x01(\r\x12$\n\tdata_type\x18\x02 \x01(\x0b\x32\x11.oi.papi.DataType\"!\n\x12\x44\x61taTypeGetRequest\x12\x0b\n\x03ids\x18\x01 \x03(\t\"Q\n\x13\x44\x61taTypeGetResponse\x12\x13\n\x0bstatus_code\x18\x01 \x01(\r\x12%\n\ndata_types\x18\x02 \x03(\x0b\x32\x11.oi.papi.DataType\"*\n\x13\x44\x61taTypeListRequest\x12\x13\n\x0bsearch_text\x18\x01 \x01(\t\"R\n\x14\x44\x61taTypeListResponse\x12\x13\n\x0bstatus_code\x18\x01 \x01(\r\x12%\n\ndata_types\x18\x02 \x03(\x0b\x32\x11.oi.papi.DataType2\xdf\x01\n\x0b\x44\x61taTypeAPI\x12I\n\x06\x63reate\x12\x1e.oi.papi.DataTypeCreateRequest\x1a\x1f.oi.papi.DataTypeCreateResponse\x12@\n\x03get\x12\x1b.oi.papi.DataTypeGetRequest\x1a\x1c.oi.papi.DataTypeGetResponse\x12\x43\n\x04list\x12\x1c.oi.papi.DataTypeListRequest\x1a\x1d.oi.papi.DataTypeListResponseb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'data_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DATATYPE._serialized_start=28
  _DATATYPE._serialized_end=135
  _DATATYPECREATEREQUEST._serialized_start=137
  _DATATYPECREATEREQUEST._serialized_end=232
  _DATATYPECREATERESPONSE._serialized_start=234
  _DATATYPECREATERESPONSE._serialized_end=317
  _DATATYPEGETREQUEST._serialized_start=319
  _DATATYPEGETREQUEST._serialized_end=352
  _DATATYPEGETRESPONSE._serialized_start=354
  _DATATYPEGETRESPONSE._serialized_end=435
  _DATATYPELISTREQUEST._serialized_start=437
  _DATATYPELISTREQUEST._serialized_end=479
  _DATATYPELISTRESPONSE._serialized_start=481
  _DATATYPELISTRESPONSE._serialized_end=563
  _DATATYPEAPI._serialized_start=566
  _DATATYPEAPI._serialized_end=789
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: visualization.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13visualization.proto\x12\x15oi.papi.visualization\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xad\x05\n\rVisualization\x12\n\n\x02id\x18\x01 \x01(\t\x12\x16\n\x0e\x63omputation_id\x18\x02 \x01(\t\x12\x1d\n\x15result_observation_id\x18\x03 \x01(\t\x12\x37\n\x04type\x18\x04 \x01(\x0e\x32).oi.papi.visualization.Visualization.Type\x12\x39\n\x05state\x18\x05 \x01(\x0e\x32*.oi.papi.visualization.Visualization.State\x12)\n\x08metadata\x18\x06 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x44\n\x0b\x61\x63\x63\x65ss_info\x18\x07 \x01(\x0b\x32/.oi.papi.visualization.Visualization.AccessInfo\x12.\n\ncreated_on\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_on\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x1aP\n\nAccessInfo\x12\x14\n\x0curl_template\x18\x01 \x01(\t\x12,\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"A\n\x04Type\x12\x10\n\x0cUNKNOWN_TYPE\x10\x00\x12\x0c\n\x08STANDARD\x10\x01\x12\x0b\n\x07SPATIAL\x10\x02\x12\x0c\n\x08TEMPORAL\x10\x03\"\x7f\n\x05State\x12\x11\n\rUNKNOWN_STATE\x10\x00\x12\x07\n\x03NEW\x10\x01\x12\n\n\x06QUEUED\x10\x02\x12\x0e\n\nVALIDATING\x10\x03\x12\t\n\x05\x45MPTY\x10\x04\x12\x0c\n\x08UPLOADED\x10\x05\x12\n\n\x06\x46\x41ILED\x10\x06\x12\x0c\n\x08\x44\x45LETING\x10\x07\x12\x0b\n\x07\x44\x45LETED\x10\x08\"9\n\x17VisualizationGetRequest\x12\x1e\n\x16result_observation_ids\x18\x01 \x03(\t\"m\n\x18VisualizationGetResponse\x12\x13\n\x0bstatus_code\x18\x01 \x01(\r\x12<\n\x0evisualizations\x18\x02 \x03(\x0b\x32$.oi.papi.visualization.Visualization2z\n\x10VisualizationApi\x12\x66\n\x03get\x12..oi.papi.visualization.VisualizationGetRequest\x1a/.oi.papi.visualization.VisualizationGetResponseb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'visualization_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _VISUALIZATION._serialized_start=110
  _VISUALIZATION._serialized_end=795
  _VISUALIZATION_ACCESSINFO._serialized_start=519
  _VISUALIZATION_ACCESSINFO._serialized_end=599
  _VISUALIZATION_TYPE._serialized_start=601
  _VISUALIZATION_TYPE._serialized_end=666
  _VISUALIZATION_STATE._serialized_start=668
  _VISUALIZATION_STATE._serialized_end=795
  _VISUALIZATIONGETREQUEST._serialized_start=797
  _VISUALIZATIONGETREQUEST._serialized_end=854
  _VISUALIZATIONGETRESPONSE._serialized_start=856
  _VISUALIZATIONGETRESPONSE._serialized_end=965
  _VISUALIZATIONAPI._serialized_start=967
  _VISUALIZATIONAPI._serialized_end=1089
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from terrascope_api.models import result_pb2 as result__pb2


class ResultApiStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.get = channel.unary_unary(
                '/oi.papi.ResultApi/get',
                request_serializer=result__pb2.ResultGetRequest.SerializeToString,
                response_deserializer=result__pb2.ResultGetResponse.FromString,
                )
        self.export = channel.unary_unary(
                '/oi.papi.ResultApi/export',
                request_serializer=result__pb2.ResultExportRequest.SerializeToString,
                response_deserializer=result__pb2.ResultExportResponse.FromString,
                )


class ResultApiServicer(object):
    """Missing associated documentation comment in .proto file."""

    def get(self, request, context):
        """
        Retrieve the Results for the specified AlgorithmComputation that match the specified filters.
        Results cannot be queried for an AnalysisComputation.
        Instead, pass in each algorithm_computation_id in the Analysis to retrieve the Results.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def export(self, request, context):
        """
        Export the more detailed version of Results.
        This endpoint accepts 1 or more algorithm_computation_ids and packages the results together into a single downloadable zip file placed in S3.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ResultApiServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'get': grpc.unary_unary_rpc_method_handler(
                    servicer.get,
                    request_deserializer=result__pb2.ResultGetRequest.FromString,
                    response_serializer=result__pb2.ResultGetResponse.SerializeToString,
            ),
            'export': grpc.unary_unary_rpc_method_handler(
                    servicer.export,
                    request_deserializer=result__pb2.ResultExportRequest.FromString,
                    response_serializer=result__pb2.ResultExportResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'oi.papi.ResultApi', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ResultApi(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/oi.papi.ResultApi/get',
            result__pb2.ResultGetRequest.SerializeToString,
            result__pb2.ResultGetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def export(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/oi.papi.ResultApi/export',
            result__pb2.ResultExportRequest.SerializeToString,
            result__pb2.ResultExportResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2017-2021 Orbital Insight Inc., all rights reserved.
# Contains confidential and trade secret information.
# Government Users:  Commercial Computer Software - Use governed by
# terms of Orbital Insight commercial license agreement.


# TODO: figure out reuse sync client. async and sync client almost same
import os

import grpc
import json
from terrascope_api import models
from terrascope_api import stubs
from terrascope_api.async_interceptors import (
    ContextInjectorUnaryUnary, ContextInjectorStreamUnary, ContextInjectorUnaryStream, ContextInjectorStreamStream
)

API_TOKEN = 'TERRASCOPE_API_TOKEN'


class TerrascopeAsyncClient:
    def __init__(self, oi_papi_url, port=443, secure=True, api_token=None):
        self.api = TerrascopeAsyncApi(oi_papi_url, port, secure, api_token=api_token)
        self.models = models


class TerrascopeAsyncApi:
    def __init__(self, oi_papi_url, port=443, secure=True, api_token=None):
        self._oi_papi_url = oi_papi_url
        self._port = port
        if api_token is None:
            api_token = os.environ.get(API_TOKEN)
            if api_token is None:
                homedir = os.environ.get('HOME', '/')
                token_filename = f'{homedir}/.orbital/api_token'
                try:
                    with open(token_filename) as fd:
                        api_token = fd.read()
                except FileNotFoundError:
                    pass
        if api_token is not None:
            api_token = api_token.strip()
        self.api_token = api_token
        if not secure and api_token is not None:
            raise ValueError('API token cannot be passed over an unsecure connection')
            # grpc will not transmit the Authorization header over an unsecured channel, which
            #  makes sense since it would be easy to hijack
        self.context_injecting_interceptors = [
            # materialize distributed context from request headers
            ContextInjectorUnaryUnary(), ContextInjectorUnaryStream(),
            ContextInjectorStreamUnary(), ContextInjectorStreamStream()
        ]
        self.options = [('grpc.max_send_message_length', -1),
                        ('grpc.max_receive_message_length', -1)]
        service_config_json = json.dumps({
            "methodConfig": [{
                "name": [{}],  # Apply retry to all methods by using [{}]
                "retryPolicy": {
                    "maxAttempts": 5,
                    "initialBackoff": "1.0s",
                    "maxBackoff": "60s",
                    "backoffMultiplier": 2,
                    "retryableStatusCodes": ["UNAVAILABLE"],
                },
            }]
        })
        self.options.append(("grpc.service_config", service_config_json))
        self._channel = self._get_secure_channel() if secure else self._get_insecure_channel()
        self.algorithm = stubs.algorithm.AlgorithmApiStub(self._channel)
        self.algorithm_version = stubs.algorithm_version.AlgorithmVersionApiStub(self._channel)
        self.algorithm_config = stubs.algorithm_config.AlgorithmConfigApiStub(self._channel)
        self.aoi = stubs.aoi.AOIApiStub(self._channel)
        self.aoi_collection = stubs.aoi_collection.AOICollectionApiStub(self._channel)
        self.aoi_version = stubs.aoi_version.AOIVersionApiStub(self._channel)
        self.aoi_transaction = stubs.aoi_transaction.AOITransactionApiStub(self._channel)
        self.algorithm_computation = stubs.algorithm_computation.AlgorithmComputationApiStub(self._channel)
        self.analysis = stubs.analysis.AnalysisApiStub(self._channel)
        self.analysis_version = stubs.analysis_version.AnalysisVersionApiStub(self._channel)
        self.credit = stubs.credit.CreditApiStub(self._channel)
        self.result = stubs.result.ResultApiStub(self._channel)
        self.analysis_config = stubs.analysis_config.AnalysisConfigApiStub(self._channel)
        self.analysis_computation = stubs.analysis_computation.AnalysisComputationApiStub(self._channel)
        self.system = stubs.system.SystemApiStub(self._channel)
        self.toi = stubs.toi.TOIApiStub(self._channel)
        self.permission = stubs.permission.PermissionApiStub(self._channel)
        self.user = stubs.user.UserApiStub(self._channel)
        self.user_collection = stubs.user_collection.UserCollectionApiStub(self._channel)
        self.token = stubs.token.TokenApiStub(self._channel)
        self.visualization = stubs.visualization.VisualizationApiStub(self._channel)

        self.data_source = stubs.data_source.DataSourceAPIStub(self._channel)
        self.data_type = stubs.data_type.DataTypeAPIStub(self._channel)

    def _get_insecure_channel(self):
        return grpc.aio.insecure_channel(f"{self._oi_papi_url}:{self._port}",
                                         options=self.options,
                                         interceptors=self.context_injecting_interceptors)

    def _get_secure_channel(self):
        ssl_creds = grpc.ssl_channel_credentials(root_certificates=None, private_key=None, certificate_chain=None)
        # note: with all defaults, gRPC will search for cert as described here:
        #  https://github.com/grpc/grpc/blob/7a63bd5407d5e14b30f19a5aaf4b6cd1b80f00e1/include/grpc/grpc_security.h#L287
        if self.api_token is not None:
            token_creds = grpc.access_token_call_credentials(self.api_token)
            creds = grpc.composite_channel_credentials(ssl_creds, token_creds)
        else:
            creds = ssl_creds
        return grpc.aio.secure_channel(f"{self._oi_papi_url}:{self._port}",
                                       creds,
                                       options=self.options,
                                       compression=None,
                                       interceptors=self.context_injecting_interceptors)

    # TODO: async destructor
    def __adel__(self):
        self._channel.close()
